from collections import OrderedDict
from interpreter13 import Lexer, Parser, NodeVisitor, BuiltinTypeSymbol, VarSymbol


class SemanticAnalyzer(NodeVisitor):  # 添加语义分析器
    def __init__(self):
        self.symbol_table = SymbolTable()  # 创建符号表

    def visit_Program(self, node):  # 添加与访问变量声明相关的访问方法
        self.visit(node.block)

    def visit_Block(self, node):  # 添加与访问变量声明相关的访问方法
        for declaration in node.declarations:
            self.visit(declaration)
        self.visit(node.compound_statement)

    def visit_VarDecl(self, node):  # 添加访问变量声明的方法（1.遍历变量声明节点）
        symbol_type = BuiltinTypeSymbol('INTEGER')  # 创建内置类型对象
        self.symbol_table.insert(symbol_type)  # 内置类型符号对象添加到符号表

        var_name = node.var_node.name  # 获取变量名称（2.收集变量信息）
        var_symbol = VarSymbol(var_name, symbol_type)  # 创建变量符号对象（2.收集变量信息）
        self.symbol_table.insert(var_symbol)  # 添加变量符号对象到符号表（3.存入存储区）

    def visit_Compound(self, node):  # 添加与访问变量声明相关的访问方法
        for child in node.children:
            self.visit(child)

    def visit_NoOperator(self, node):  # 添加与访问变量声明相关的访问方法
        pass


class SymbolTable:
    def __init__(self):
        self._symbols = OrderedDict()

    def __str__(self):
        symtab_header = '符号表中的内容：'
        lines = [symtab_header, '-' * len(symtab_header) * 2]
        lines.extend([f'{key:8}: {value}' for key, value in self._symbols.items()])
        s = '\n'.join(lines)
        return s

    __repr__ = __str__

    def insert(self, symbol):
        print(f'存入：{symbol}')
        self._symbols[symbol.name] = symbol


if __name__ == '__main__':
    text = '''
        program SymTab2;
           var x, y : integer;
        
        begin
        
        end.
    '''
    lexer = Lexer(text)
    parser = Parser(lexer)
    tree = parser.parser()
    semantic_analyzer = SemanticAnalyzer()
    semantic_analyzer.visit(tree)
    print(semantic_analyzer.symbol_table)
